/**
* Creates an instance of Field
*
* @class
* @name Field
* @param {ZhUtil} ZhUtil - An instance of zhUtil.
* @param {TemplateManager} templateManager - An instance of TemplateManager
* @param {DecodeValueFieldService} DecodeValueFieldService - An instance of DecodeValueFieldService
*/
var Field = function(ZhUtil, templateManager, DecodeValueFieldService, validateEngine) {
	var DEFAULT_VALUES = {
		valueToTrue: 'Y',
		valueToFalse: 'N',
		labelToTrue: 'Yes',
		labelToFalse: 'No',
		'class': 12,
		showOnForm: true,
		showOnView: true,
		showOnList: false,
		showOnFilter: false,
		showCheckbox: true,
		showSeparatorButton: true,
		showSignalButton: true,
		isVisible: true,
		aclIsVisible: true,
		aclReadOnly: false,
		readOnly: false,
		readOnlyOnEdit: false,
		translate: false,
		sequenceInForm: 10,
		events: [],
		isVisibleOnReport: true
	};
	var _validationEngine;

	var self = this;

	/**
	* Reloads the field
	* @function reload
	* @memberof Field
	* @instance
	* @param {Object[]} [addictionalFilter] - An object of filters to be applied when reloading the field
	*/
	this.reload = function(addictionalFilters) {
		if (this.dataSource) {
			var filters = Util.parseToArray(this.dataSourceFilter);
			if (Util.isArray(addictionalFilters)) {
				filters = filters.concat(addictionalFilters);
			}
			this.dataSource.setCurrentPage(1);
			return this.dataSource.load(filters, this.dataSource.getCurrentPage(), this.itemsPerPage);
		} else {
			return {
				then: function(fullfil) {
					fullfil();
				}
			};
		}
	};

	/**
	* Verify if field has readOnly property set as true
	* @function isReadOnly
	* @memberof Field
	* @instance
	* @param
	* @returns {Boolean} True if the field has the readOnly property
	*/
	this.isReadOnly = function() {
		return this.readOnly || this.aclReadOnly || (this.readOnlyOnEdit && !this.widget.isNewRow());
	};

	/**
	* Empty method
	* @function show
	* @memberof Field
	* @instance
	*/
	this.show = function() {

	};
	/**
	* Empty method
	* @function hide
	* @memberof Field
	* @instance
	*/
	this.hide = function() {

	};
	/**
	* Get the widget that owns the field
	* @function getParent
	* @memberof Field
	* @instance
	* @returns {Widget} Parent widget
	*/
	this.getParent = function() {
		return this.widget;
	};
	/**
	* Get the value of the field or defines a new one
	* @function value
	* @memberof Field
	* @instance
	* @param {String|Object} [value] - (Use only to set a new value to the field) Value to be set in the field
	* @returns {String|Object} If there is no value to set, returns the current value of the field
	*/
	this.value = function(value) {
		if (typeof value !== "undefined") {
			this.widget.currentRow[this.name] = value;
			templateManager.updateTemplate();
		} else {
			return this.widget.currentRow[this.name];
		}
	};
	/**
	* Clears the value on the field
	* @function clearValue
	* @memberof Field
	* @instance
	* @param
	*/
	this.clearValue = function() {
		this.widget.currentRow[this.name] = "";
		templateManager.updateTemplate();
	};
	/**
	* Changes the current row of the widget
	* @function setCurrentRow
	* @memberof Field
	* @instance
	* @param {Object} row - data to be set in the current row of the field
	* @returns {Event} Returns an event
	*/
	this.setCurrentRow = function(row) {
		var event;
		if (this.beforeMoveRow) {
			event = this.beforeMoveRow();
		}
		if (!event || !event.isDefaultPrevented()) {
			this.currentRow = row;
			if (this.afterMoveRow) {
				this.afterMoveRow();
			}
		}
		return event;
	};
	/**
	* Sets a value on the field
	* @function setValue
	* @memberof Field
	* @instance
	* @param {String} value - Value to be set in the current row
	*/
	this.setValue = function(value) {
		this.widget.currentRow[this.name] = value;
	};
	/**
	* Reset the value of the field to default
	* @function applyDefaultValue
	* @memberof Field
	* @instance
	*/
	this.applyDefaultValue = function() {
		if (this.widget.currentRow) {
			this.widget.currentRow[this.name] = typeof this.defaultValue === "undefined" ? "" : ZhUtil.evaluate(this.defaultValue);
		}
	};
	/**
	* Changes the field sequence
	* @function setSequence
	* @memberof Field
	* @instance
	* @param {Number} sequence - New sequence value
	* @param {Boolean} override - If true, overrides last sequence value
	*/
	this.setSequence = function(sequence, override) {
		this.sequenceInList = (Util.isEmptyOrBlank(this.sequenceInList) || override) ? sequence : this.sequenceInList;
	};
	/**
	* Returns the model value
	* @function modelValue
	* @memberof Field
	* @instance
	*/
	this.modelValue = function() {
		return DecodeValueFieldService.model(this, this.value());
	};
	/**
	* Verifies if the field value matches the validations
	* @function validate
	* @memberof Field
	* @instance
	* @param {String|Object} value - Value of the field
	* @param {Function} callback - Function to receive the data of the errors
	* @returns {String|Object} Returns the value of the field after validation
	*/
	this.validate = function(value, callback) {
		var field = this;
		var widget = this.widget;
		var fieldName = this.name;
		var fieldLabel = this.label;
		var validations = this.validations;
		var errors = [];

		widget.currentRow.$error = widget.currentRow.$error || {};
		widget.currentRow.$error[fieldName] = [];

		if (this.isVisible && this.aclIsVisible && validations) {
			var validationNameToValidationFunction = validateEngine.getMappedValidations();
			Object.keys(validations).forEach(function (validation) {
				var validateResult = validationNameToValidationFunction[validation](value, Object.assign({}, validations[validation], { field: field }));
				if (!validateResult.valid) {
					widget.currentRow.$error[fieldName].push(validateResult);
					var errorObj = {
						fieldName: fieldName,
						fieldLabel: fieldLabel,
						errorMessage: validateResult.message,
						valid: validateResult.valid
					};

					errors.push(errorObj);
				}
			});
		}

		if(callback){
			callback(errors);
		}
		return value;
	};

	var _setDefaultValuesField = (function(field) {
		Util.extend(field, DEFAULT_VALUES);
		Util.extend(field, new Templatable());
	})(this);
};